/*
Pro pouziti sbernice I2C, na ktere bezi I/O expander MCP23008
a pamet AT24c256

@author = Vojtech Hajek
DP Casomerna jednotka pro automobilovy sprint
prace s pameti inspirovana
https://arduino8.webnode.cz/news/lekce-18-arduino-a-externi-eeprom-pres-i2c/
*/

#include <Arduino.h>
#include <Wire.h>
#include "Adafruit_MCP23008.h"
#include "beam.h"
#include "I2C.h"
#include "tisr.h"
#include "myBluetooth.h"

Adafruit_MCP23008 mcp;

#define MCPAddress 20
#define IODIR 0x00
#define memAdr 0x50 //adresa I2C pameti
//cas, ktery se spolu s cislem preruseni bude okladat do eeprom
//do pameti at24c256 -> 32768bajtu,
//celkem 4 bajty jsou potreba pro zapis casu a preruseni /jeste 2bity zbudou (pro pripadne rozsireni preruseni)
//tttt tttt tttt tttt tttt tttt tttx iiii cas je ulozeny v tisicinach a k tomu prictene cislo preruseni
//32768/4 = 8192, 4 bajty schovany pro zapis identifikatoru (poctu mista v eeprom pripadne dalsich) takze muzeme ulozit dost casu
int writeAddress = 4; //startovni adesa, kam bude zapisovat do pameti, TODO lepe vyresit nactenim z pameti pri startu ESP
int startCounter;     //pro prvni nacteni poctu casu v pameti
int timeCounter;      //pocet casu v eeprom,

const int MAX_TIMES = 8191; //8192 - 1 bit pro zapis dat, zustanou 3bajty volne
bool dataRead = false;      //pokud jsou ctena data ignoruj vypis baterie a zapis do pameti
void rLightOn(int lane)
{ //1 leva draha cokoliv jineho prava
    lane == 1 ? digitalWrite(RL, HIGH) : digitalWrite(RR, HIGH);
}
void rLightOff(int lane)
{
    lane == 1 ? digitalWrite(RL, LOW) : digitalWrite(RR, LOW);
}
void lightOff()
{
    for (int i = 0; i < 8; i++)
    {
        mcp.digitalWrite(i, LOW);
    }
    rLightOff(1);
    rLightOff(0);
}
void lightOn()
{
    for (int i = 0; i < 8; i++)
    {
        mcp.digitalWrite(i, HIGH);
    }
}
void lightDo(int code, bool up)
{
    // Serial.print("kod: ");
    // Serial.println(code);
    switch (code)
    {
    case 99: //rozsvit nebo zhasni vsechno
        up ? lightOn() : lightOff();
        up ? rLightOn(1) : rLightOff(1);
        up ? rLightOn(0) : rLightOff(0);
        break;
    case 98: //rozsvit nebo zhasni vsechny rude
        up ? rLightOn(1) : rLightOn(1);
        up ? rLightOn(0) : rLightOff(0);
        if (up)
        {
            Serial.println("RL1");
            Serial.println("RR1");
            writeBT("RL1");
            writeBT("RR1");
        }
        else
        {
            Serial.println("RL0");
            Serial.println("RR0");
            writeBT("RL0");
            writeBT("RR0");
        }
        break;
    default:
        if (code == RL || code == RR)
        {
            if (up)
                code == RL ? rLightOn(1) : rLightOn(0);
            else
                code == RR ? rLightOff(1) : rLightOff(0);
        }
        else
            up ? mcp.digitalWrite(code, HIGH) : mcp.digitalWrite(code, LOW);

        break;
    }
}
bool isDataRead(){
    return dataRead;
}
/**nacti bajt po bajtu z d
 * disk - adresa i2c pameti
 * ad - adresa odkud se bude cist
 * 
 * */
byte readEEPROM(int disk, unsigned int ad)
{
    byte rdata = 0xFF;
    Wire.beginTransmission(disk);
    Wire.write((int)(ad));        // odešle vyšší byte
    Wire.write((int)(ad & 0xFF)); // odešle nižší byte
    Wire.endTransmission();
    Wire.requestFrom(disk, 1);
    if (Wire.available())
    {
        rdata = Wire.read();
        //  Serial.print("rwdata: "); Serial.println(rdata);
        //  Serial.print("adresa:\t");Serial.println(eeaddress);
        //  Serial.print("rdata:\t");Serial.println(rdata);
    }
    return rdata;
}
void initLight()
{
    mcp.begin();
    mcp.pinMode(Y1, OUTPUT);
    mcp.pinMode(Y2, OUTPUT);
    mcp.pinMode(Y3, OUTPUT);
    mcp.pinMode(NL, OUTPUT);
    mcp.pinMode(FL, OUTPUT);
    mcp.pinMode(NR, OUTPUT);
    mcp.pinMode(FR, OUTPUT);
    mcp.pinMode(G, OUTPUT);

    pinMode(RL, OUTPUT);
    pinMode(RR, OUTPUT);
}
void initEEPROM()
{
    Wire.begin();
    initLight();
    Wire.beginTransmission(memAdr);
    //prace s adresou - dva bajty rozdeleny pro zapis po bajtu
    Wire.write((int)0);
    Wire.write((int)(0 & 0xFF));
    Wire.endTransmission();

    Wire.requestFrom(memAdr, 2);
    byte data[2] = {0xFF};
    for (int i = 0; i < 2; i++)
    {
        data[i] = readEEPROM(memAdr, i);
        // Serial.print("init data[i]: ");
        // Serial.print(i);
        // Serial.print(", ");
        // Serial.println(data[i]);
    }
    startCounter = data[0] | (data[1] << 8); //index dalsiho casu
    timeCounter = startCounter + 1;          //casu v pameti pri initu je jako startcounter, je to nasledujici cas
                                             //   Serial.print("Init EEPROM - startCounter: ");
                                             //   Serial.println(startCounter);
    writeAddress = (startCounter * 4 + 4) % MAX_TIMES;
    String msg = "E";
    msg += startCounter;
    writeBT(msg);
    Serial.println(msg);
    //   Serial.print("Init EEPROM - writeAddress: ");
    //   Serial.println(writeAddress);
    //  timeCounter = 210;
}
void updateIndex(int t)
{
    byte bajt0 = t;
    byte bajt1 = t >> 8;
    Wire.beginTransmission(memAdr);
    Wire.write((int)(0));        //MSB
    Wire.write((int)(0 & 0xFF)); //LSB
    Wire.write(bajt0);
    Wire.endTransmission();
    delay(10);

    Wire.beginTransmission(memAdr);
    Wire.write((int)(1));        //MSB
    Wire.write((int)(1 & 0xFF)); //LSB
    Wire.write(bajt1);
    Wire.endTransmission();
    delay(10);
}

void deleteEEPROM()
{
    writeAddress = 4;
    timeCounter = 0;
    startCounter = 1;
    updateIndex(timeCounter);
    writeBT("E0");
    Serial.println("E0");
    //pamet nevymaze, jen nastavi pointery
}
void checkEEPROM()
{
    //kdyz je v pameti 8191 casu, tak to resetuj
    // a budeme pretykat - FIFO
    if (timeCounter == MAX_TIMES)
    {                     // nastav na 0
        writeAddress = 4; // nastav na 4 bajt v pameti
    }
    if (timeCounter > MAX_TIMES)
    {
        writeAddress = (timeCounter % MAX_TIMES) * 4 + 4;
    }
}

void writeEEPROM(int ad, long v)
{
    byte a[4] = {0xFF};
    a[0] = (v & 0xFF);
    a[1] = ((v >> 8) & 0xFF);
    a[2] = ((v >> 16) & 0xFF);
    a[3] = ((v >> 24) & 0xFF);

    for (int i = 0; i < 4; i++)
    {
        Wire.beginTransmission(memAdr);
        Wire.write(ad + i);
        Wire.write((ad + i) & 0xFF);
        Wire.write(a[i]);
        Wire.endTransmission();
        delay(10);
        // Serial.print("adresa: "); Serial.print(ad + i); Serial.print(" :");
        // Serial.println(a[i], BIN);
    }

    updateIndex(timeCounter);
    writeAddress += 4;
}

void getData() //ma vratit cas a cislo preruseni
{
    dataRead = true; //pokud by se mel ukladat cas do pameti, at nedojde ke kolizi a vypisum na linku
    int x = MAX_TIMES;
    int y = 0;
    if (timeCounter == 0)
    {
        Serial.println("E0");
        writeBT("E0");
        return;
    }

    if (timeCounter < MAX_TIMES)
    {
        x = timeCounter + 1;
        y = 1;
    }
    else
    {
        y = startCounter;
        x = timeCounter + startCounter;
    }

    for (int j = y; j < x; j++)
    {
        // Serial.print("j: ");
        // Serial.println(j);

        byte a[4] = {0xFF};

        if (j < MAX_TIMES)
        {
            // Serial.print("j<max times");
            a[0] = readEEPROM(0x50, j * 4);
            a[1] = readEEPROM(0x50, j * 4 + 1);
            a[2] = readEEPROM(0x50, j * 4 + 2);
            a[3] = readEEPROM(0x50, j * 4 + 3);

            //      Serial.print("adresa: "); Serial.print(j * 4); Serial.print(" :"); Serial.println(a[0], BIN);
            //      Serial.print("adresa: "); Serial.print(j * 4 + 1); Serial.print(" :"); Serial.println(a[1], BIN);
            //      Serial.print("adresa: "); Serial.print((j) * 4 + 2); Serial.print(" :"); Serial.println(a[2], BIN);
            //      Serial.print("adresa: "); Serial.print((j) * 4 + 3); Serial.print(" :"); Serial.println(a[3], BIN);
        }
        else
        { //aby vynechal nulte 4 bity
            //      Serial.print("j>max times, j: ");
            //      Serial.println(j);
            a[0] = readEEPROM(0x50, (j % MAX_TIMES) * 4 + 4);
            //Serial.print("adresa: "); Serial.print((j % MAX_TIMES) * 4 + 4); Serial.print(" :"); Serial.println(a[0], BIN);
            a[1] = readEEPROM(0x50, (j % MAX_TIMES) * 4 + 1 + 4);
            //  Serial.print("adresa: "); Serial.print((j % MAX_TIMES) * 4 + 5); Serial.print(" :"); Serial.println(a[1], BIN);
            a[2] = readEEPROM(0x50, (j % MAX_TIMES) * 4 + 2 + 4);
            //  Serial.print("adresa: "); Serial.print((j % MAX_TIMES) * 4 + 6); Serial.print(" :"); Serial.println(a[2], BIN);
            a[3] = readEEPROM(0x50, (j % MAX_TIMES) * 4 + 3 + 4);
            //  Serial.print("adresa: "); Serial.print((j % MAX_TIMES) * 4 + 7); Serial.print(" :"); Serial.println(a[3], BIN);
        }

        unsigned long resx = a[3] << 24 | a[2] << 16 | a[1] << 8 | a[0];
        //
        int isrN = resx & 0x7; // maskovani ziskame cislo preruseni
        resx = resx >> 4;
        convertMillisToTime(resx, isrN);
    }
    dataRead = false; //povol zapis do pameti
}

void saveData(int time)
{
    checkEEPROM();
    writeEEPROM(writeAddress, time);
    timeCounter++;
    String msg = "E";
    msg += timeCounter;
    //posli na kanaly ze je v pameti o cas vice
    writeBT(msg);
    Serial.println(msg);
}