/*
Pro vyhodnocovani dat ze senzoru priblizovani
tak je zde tvoren impuls pro externi casomernou jednotku po rozsviceni zelene

@author = Vojtech Hajek
DP Casomerna jednotka pro automobilovy sprint
*/

#include <Arduino.h>
#include "myBluetooth.h"
#include "myEspNow.h"
#include "beam.h"
#include "parser.h"
#include "I2C.h"
#include "tisr.h"

#define OUT 13

const int LEFT = 1;
const int RIGHT = 2;

const int CARS = 0;
const int DRAG = 1;
const int NONE = 2;
int druhStartu = CARS;

const int sFL = 1;
const int sNL = 2;
const int sFR = 4;
const int sNR = 8;

int startPosition = 0;
bool nlEspFirst = false; //pro prvni prejezd do fl, tam to blikne kdyz nema
struct Light
{                      //obecna struktura pro svetla
    const uint8_t pin; //pin, na kterem je pripojeno k esp32
    int stav;          //stav... 0 - preruseno, 1 - nepreruseno
    bool pre;          //predchozi stav
    bool change;       //je zmena pro update vypisu?
    int debMilPre;
};
struct nearLight
{ //zpresnena struktura pro ovladani a vyhodnocovani NL/R
    struct Light b;
    bool on;           //je zapnuta
    bool espNowChange; //zprava pro espMessageZmena
    bool blink;        //jestli zapinam/vypinam vysilac NL/R
    int espMessage;
    int onLightPreTime;
};
nearLight nl = {
    {nlPin, 1, true, true, 0}, false, false, false, 0};
Light fl = {flPin, 1, true, true, 0};
nearLight nr = {
    {nrPin, 1, true, true, 0}, false, false, false, 0};
Light fr = {frPin, 1, true, true, 0};

const int startN = 1;      //zapni vysilac
const int stopN = 2;       //vypni vysilac
const int tBlink = 300;    //cas pro blikani vysilace 0.3-0.5 jsou ok, lidske oko je nezpozoruje
const int tBlinkDeb = 250; //pro omezeni vypisu na seriovou linku, at se nezblazni na vypisech cpu

bool sequence = false;
bool first = true;

bool dragNL = false;
bool dragNR = false;
bool dragFL = false;
bool dragFR = false;
// unsigned long prodleva = 1000;
// unsigned long ty = 500;
// unsigned long deltaTg = 0;
// int b = 1; //pro kroky zlute a kvuli preskoceni aby se to nezacyklilo
bool isSequence()
{
    return sequence;
}
void setSequence(bool s)
{
    sequence = s;
}
bool isFirst()
{
    return first;
}
void setFirst(bool f)
{
    first = f;
}
void initBeam()
{
    pinMode(nl.b.pin, INPUT);
    pinMode(fl.pin, INPUT);
    pinMode(nr.b.pin, INPUT);
    pinMode(fr.pin, INPUT_PULLUP);
    pinMode(OUT, OUTPUT);

    attachInterrupt(nl.b.pin, isrNL, CHANGE);
    attachInterrupt(fl.pin, isrFL, CHANGE);
    attachInterrupt(nr.b.pin, isrNR, CHANGE);
    attachInterrupt(fr.pin, isrFR, CHANGE);

    startPosition = 0; //musi byt 0 aby to vsechno fungovalo spravne pro obe drahy
    druhStartu = getBeamType();
}
void setBeamType(int type)
{
    druhStartu = type;
}
bool getStartPosition()
{
    return dragNL && dragFL && dragNR && dragFR;
}
void setStartPosition(int s, bool up)
{
    up ? startPosition += s : startPosition -= s;
    // Serial.print("startPos: ");
    // Serial.println(startPosition);
}
void setStartPosition(int s)
{
    dragFR=false;
    dragNL=false;
    dragNR=false;
    dragFL=false;
    // startPosition = s;
    // Serial.print("startPosSA: ");
    // Serial.println(startPosition);
}

void IRAM_ATTR isrFL()
{
    fl.debMilPre = millis();

    int i = digitalRead(fl.pin);
    if (i == 0 && !fl.pre)
    {
        fl.stav = 0;
        fl.pre = true;
        fl.change = true;
        nl.espMessage = startN;
        // nl.blink = true; //zapinam blikani pro NL foto
        // nl.on = true;
        // nl.onLightPreTime = fl.debMilPre; //casova znacka pro blikac
        nl.espNowChange = true;
    }
    if (i == 1 && fl.pre)
    {
        if (nl.b.stav == 0)
        {
            fl.stav = 1;
            fl.pre = false;
            fl.change = true;
        }
        else
        {
            fl.stav = 1; //nepreruseno
            fl.pre = false;
            fl.change = true;
            nl.espMessage = stopN; //zprava pro espnow
            // nl.blink = false;      //vypni blikani pro NL foto
            // nl.on = false;
            nl.espNowChange = true;
            // nl.onLightPreTime = fl.debMilPre;
        }
    }
}

void IRAM_ATTR isrNL()
{
    nl.b.debMilPre = millis();

    if (nl.blink && nl.on && nl.onLightPreTime - fl.debMilPre > 100)
    { //kdyz je povolene blikani a zaroven je vysilac pusteny
        int i = digitalRead(nl.b.pin);

        if (i == 1 && !nl.b.pre)
        {
            nl.b.stav = 1; //neprerusena NL
            nl.b.pre = true;
            nl.b.change = true;
            if (fl.stav == 1) //je neprerusena FL
            {
                nl.espMessage = stopN; //zprava pro espnow, muzu vypnout
             //   nl.blink = false;      //vypni blikani pro NL foto
             //   nl.on = false;
                nl.espNowChange = true;
                // nl.onLightPreTime = nl.b.debMilPre;
            }

            // setStartPosition(sNL, true);
            return;
        }
        if (i == 0 && nl.b.pre)
        {
            nl.b.stav = 0;
            nl.b.pre = false;
            nl.b.change = true;
            // startPosition += sNL;
            return;
        }
    }
}
void IRAM_ATTR isrFR()
{
    fr.debMilPre = millis();

    int i = digitalRead(fr.pin);
    if (i == 0 && !fr.pre)
    {
        fr.stav = 0;
        fr.pre = true;
        fr.change = true;
        nr.espMessage = startN;
        nr.espNowChange = true;

        // setStartPosition(sFR, true);
    }
    if (i == 1 && fr.pre)
    {
        if (nr.b.stav == 0)
        {
            fr.stav = 1;
            fr.pre = false;
            fr.change = true;
            // setStartPosition(sNR, false);
        }
        else
        {
            fr.stav = 1;
            fr.pre = false;
            fr.change = true;
            nr.espMessage = stopN; //zprava pro espnow
            // nr.blink = false;      //vypni blikani pro NL foto
            // nr.on = false;
            nr.espNowChange = true;
            // setStartPosition(sNR, false);
        }
    }
}

void IRAM_ATTR isrNR()
{
    nr.b.debMilPre = millis();

    if (nr.blink && nr.on && nl.onLightPreTime - fl.debMilPre > 100)
    { //kdyz je povolene blikani a zaroven je vysilac pusteny
        int i = digitalRead(nr.b.pin);
        if (i == 1 && !nr.b.pre)
        {
            nr.b.stav = 1; //neprerusena NL
            nr.b.pre = true;
            nr.b.change = true;
            if (fr.stav == 1) //je neprerusena fr
            {
                nr.espMessage = stopN; //zprava pro espnow
                // nr.blink = false;      //vypni blikani pro NL foto
                // nr.on = false;
                nr.espNowChange = true;
            }
            // setStartPosition(sNR, true);
        }
        if (i == 0 && nr.b.pre)
        {
            nr.b.stav = 0;
            nr.b.pre = false;
            nr.b.change = true;
        }
    }
}
void getPulse()
{
    digitalWrite(OUT, HIGH);
    delay(2);
    digitalWrite(OUT, LOW);
    // Serial.println("ping00");
}

void checkBeams()
{
    if (!dragFR || !dragNR)
    {
        startR(false);
        writeBT("RR1");
        Serial.println("RR1");
    }
    if (!dragFL || !dragNL)
    {
        startR(true);
        writeBT("RL1");
        Serial.println("RL1");
    }
    sequence = true;
}
void priblizuj()
{
    if (druhStartu != NONE)
    {
        //    Serial.print("blikani: "); Serial.println(blikani);
        if (nl.blink && millis() - fl.debMilPre > tBlinkDeb)
        { // v intervalu se pousti NL fotobunky
            int t = millis();
            if (t - nl.onLightPreTime > tBlink)
            {
                //vyres poslani zpravy pro vypnuti/zapnuti NL
                if (nl.on)
                {
                    nl.espMessage = stopN;
                    // nl.espNowChange = true;
                    espSendData(LEFT, nl.espMessage);
                    nl.espNowChange = false;
                    nl.on = false;
                    nl.onLightPreTime = t;
                    // Serial.print("L:");
                    // Serial.println(stopN);
                    //posli zpravu L0
                    //vypni vysilac
                }
                else
                {
                    nl.espMessage = startN;
                    // nl.espNowChange = true;
                    espSendData(LEFT, nl.espMessage);
                    nl.espNowChange = false;
                    nl.on = true; //zapni vysilac
                    nl.onLightPreTime = t;
                    // Serial.print("L:");
                    // Serial.println(startN);
                    delay(100); //pockej nez nabehne
                                //zrejme by slo vyresit nejako ACK zpravou od druhe strany nebo cekani bez delay
                    isrNL();    //pak ho pust
                }
                
            }
        }
        if (fl.change && millis() - fl.debMilPre > tBlinkDeb)
        { //doslo k preruseni na FL
            //je od minuleho preruseni alespon 200ms??
            //pro mensi zatizeni CPU a omezeni blikani
            // Serial.println("flch");
            //vypis
            if (druhStartu == DRAG || druhStartu == CARS)
            { //mozna neni nutne, pro drag i auta se rozsveci stejne
                if (fl.stav == 1)
                {
                    lightDo(FL, true);
                    Serial.println("FL1");
                    writeBT("FL1");
                    if (fl.pre != 1)
                        setStartPosition(sFL, false);
                    dragFL = false;
                }
                if (fl.stav == 0)
                {
                    //rozsvit
                    lightDo(FL, false);
                    Serial.println("FL0");
                    writeBT("FL0");
                    if (fl.pre != 0)
                        setStartPosition(sFL, true);
                    dragFL = true;
                    //        Serial.print("flstav: "); Serial.println(fl.stav);
                }
            }
            fl.change = false;
        }
        if (nl.b.change && nl.b.debMilPre - millis() > tBlinkDeb)
        {
            // Serial.println("nlch");
            if (druhStartu == DRAG)
            {
                // Serial.println("nlchDRAG");
                if (nl.b.stav == 0)
                {
                    lightDo(NL, false);
                    Serial.println("NL0");
                    writeBT("NL0");
                    dragNL = true;
                    setStartPosition(sNL, true);
                }
                if (nl.b.stav == 1)
                {
                    lightDo(NL, true);
                    Serial.println("NL1");
                    writeBT("NL1");
                    dragNL = false;
                    setStartPosition(sNL, false);
                }
            }
            if (druhStartu == CARS)
            {
                // Serial.println("nlchcars");
                if (nl.b.stav == 0)
                {
                    lightDo(NL, true);
                    Serial.println("NL1");
                    writeBT("NL1");
                }
                if (nl.b.stav == 1)
                {
                    lightDo(NL, false);
                    Serial.println("NL0");
                    writeBT("NL0");
                }
            }
            nl.b.change = false;
        }
    }
    if (druhStartu != NONE)
    {
        //    Serial.print("blikani: "); Serial.println(blikani);
        if (nr.blink && millis() - fr.debMilPre > tBlinkDeb)
        { // v intervalu se pousti NL fotobunky
            int t = millis();
            if (t - nr.onLightPreTime > tBlink)
            {
                //vyres poslani zpravy pro vypnuti/zapnuti NL
                if (nr.on)
                {
                    nr.espMessage = stopN;
                    // nr.espNowChange = true;
                    espSendData(RIGHT, nr.espMessage);
                    nr.espNowChange = false;
                    nr.on = false;
                    nr.onLightPreTime = t;
                }
                else
                {
                    nr.espMessage = startN;
                    // nr.espNowChange = true;
                    espSendData(RIGHT, nr.espMessage);
                    nr.espNowChange = false;
                    nr.onLightPreTime = t;
                    // Serial.println("R1espnow");
                    // Serial.println("zapnut");
                    nr.on = true;
                    delay(100);
                    isrNR();
                }
                
            }
        }
        if (fr.change && millis() - fr.debMilPre > tBlinkDeb)
        { //doslo k preruseni na FL
            //vypis
            if (druhStartu == DRAG || druhStartu == CARS)
            { //mozna neni nutne, pro drag i auta se rozsveci stejne
                if (fr.stav == 1)
                {
                    lightDo(FR, true);
                    Serial.println("FR1");
                    writeBT("FR1");
                    if (fr.pre != 1)
                        setStartPosition(sFR, false);
                    dragFR = false;
                }
                if (fr.stav == 0)
                {
                    //rozsvit
                    lightDo(FR, false);
                    Serial.println("FR0");
                    writeBT("FR0");
                    if (fr.pre != 0)
                        setStartPosition(sFR, true);
                    dragFR = true;
                }
            }
            fr.change = false;
        }
        if (nr.b.change && nr.b.debMilPre - millis() > tBlinkDeb)
        {
            // Serial.println("nrch");
            if (druhStartu == DRAG)
            {
                // Serial.println("nrchDRAG");
                if (nr.b.stav == 0)
                {
                    Serial.println("NR0");
                    writeBT("NR0");
                    lightDo(NR, false);
                    setStartPosition(sNR, true);
                    dragNR = true;
                }
                if (nr.b.stav == 1)
                {
                    lightDo(NR, true);
                    Serial.println("NR1");
                    writeBT("NR1");
                    setStartPosition(sNR, false);
                    dragNR = false;
                }
            }
            if (druhStartu == CARS)
            {
                // Serial.println("nrchcars");
                if (nr.b.stav == 0)
                {
                    lightDo(NR, true);
                    Serial.println("NR1");
                    writeBT("NR1");
                }
                if (nr.b.stav == 1)
                {
                    lightDo(NR, false);
                    Serial.println("NR0");
                    writeBT("NR0");
                }
            }
            nr.b.change = false;
        }
    }

    if (druhStartu == DRAG && first && dragNL && dragNR && dragFL && dragFR && isSA())
    {
        setDeltaTg(millis());
        first = false;
        sequence = true;
    }
    if (nl.espNowChange)
    {
        espSendData(LEFT, nl.espMessage);
        if(nl.espMessage == startN){
            nl.on = true;
            nl.blink = true;
        }
        if(nl.espMessage == stopN){
            nl.on = false;
            nl.blink = false;
        }
        nl.onLightPreTime = millis();
        nl.espNowChange = false;
    }
    if (nr.espNowChange)
    {
        espSendData(RIGHT, nr.espMessage);
        nr.espNowChange = false;
    }
}

void callISR()
{
    isrFL();
    isrFR();
    isrNL();
    isrNR();
}