/*
Pro parsovani zprav, ktere prijdou po Seriove lince nebo Bluetooth
Zaroven se vykonava valna cast kodu funkcionality
tj. rozsvecovani svetel

@author = Vojtech Hajek
DP Casomerna jednotka pro automobilovy sprint
*/

#include <Arduino.h>
#include "myBluetooth.h"
#include "I2C.h"
#include "parser.h"
#include "tisr.h"
#include "beam.h"

const int SA = 1; //start SA
const int SB = 6; //start SB
const int SC = 2;
const int S0 = 0;
const int SF = 3;
const int S1 = 4;
const int SDstart = 5; //START RG
const int SE = 7;

bool stSA = false;
bool startSG = false;

const int TMS = 0;
const int DRAG = 1;
const int NONE = 2;
int beamType = TMS; //TMS - 0, DRAG - 1, //NONE - 2
int startType = SB;
bool exitStart = true;
bool manualStart = true; //rucni start pomoci tlacitka, defaultne povoleny, aby to slo bez pripojeni k pc
bool IS = false;
bool lastSA = false;
bool lastSC = false;

int waitAfter = 200; //defaultne rozumne hodnoty aby to jelo autonomne
int green = 800;
int yellow = 500;
bool yellowProgress = true; //true pridava, false postupuje
bool yPro = false;          //je pro rozsveceni zlutych najednou
int b = 1;                  //pro SA sekvencni startovani
unsigned long deltaTg = 0;

bool startTrue = true;

int getBeamType()
{
    return beamType;
}
bool isSA()
{
    return lastSA;
}
void setSC(bool t)
{
    lastSC = true;
}
bool isSC()
{
    return lastSC;
}
bool isManualStart()
{
    return manualStart;
}
void setDeltaTg(unsigned long t)
{
    deltaTg = t;
}
void setB(int c)
{
    b = c;
}
String convertToString(char *a, size_t size)
{
    int i;
    String s = "";
    for (i = 0; i < size; i++)
    {
        s = s + a[i];
    }
    return s;
}

bool isYellowNull()
{
    return yPro;
}
//bool true leva
//bool false prava
void startR(bool lane)
{
    if (lane)
    {
        lightDo(RL, true);
        Serial.println("RL1");
        writeBT("RL1");
    }
    else
    {
        lightDo(RR, true);
        Serial.println("RR1");
        writeBT("RR1");
    }
}
void startG()
{
    lightDo(G, true);
    Serial.println("GL1");
    writeBT("GL1");
}
void stopRG()
{
    lightDo(RL, false);
    Serial.println("RL0");
    writeBT("RL0");
    lightDo(RR, false);
    Serial.println("RR0");
    writeBT("RR0");
    lightDo(G, false);
    Serial.println("GL0");
    writeBT("GL0");
}
void resetBeam(int type)
{
    switch (type)
    {
    case DRAG:
        // lightDo(B11110000);
        lightDo(NL, true);
        Serial.println("NL1");
        writeBT("NL1");
        lightDo(FL, true);
        Serial.println("FL1");
        writeBT("FL1");
        lightDo(NR, true);
        Serial.println("NR1");
        writeBT("NR1");
        lightDo(FR, true);
        Serial.println("FR1");
        writeBT("FR1");
        setBeamType(DRAG);
        break;
    case TMS:
        // lightDo(B10010000);
        lightDo(NL, false);
        Serial.println("NL0");
        writeBT("NL0");
        lightDo(FL, true);
        Serial.println("FL1");
        writeBT("FL1");
        lightDo(FR, true);
        Serial.println("FR1");
        writeBT("FR1");
        lightDo(NR, false);
        Serial.println("NR0");
        writeBT("NR0");
        setBeamType(TMS);
        break;
    default:
        lightDo(NL, false);
        Serial.println("NL0");
        writeBT("NL0");
        lightDo(FL, false);
        Serial.println("FL0");
        writeBT("FL0");
        lightDo(FR, false);
        Serial.println("FR0");
        writeBT("FR0");
        lightDo(NR, false);
        Serial.println("NR0");
        writeBT("NR0");
        setBeamType(NONE);
        break;
    }
}

void start(unsigned long t)
{
    // Serial.print("exitstart status SA/SC: ");
    // Serial.println(exitStart);
    //kdyz je yellow 0 tak rozsvit vsechno naraz a za pul vteriny zelenou
    if (isYellowNull())
    {
        if (b == 1)
        {
            lightDo(Y1, true);
            Serial.println("Y1");
            writeBT("Y11");

            lightDo(Y2, true);
            Serial.println("Y21");
            writeBT("Y21");
            // checkBeams();
            lightDo(Y3, true);
            Serial.println("Y31");
            writeBT("Y31");
            // checkBeams();
            b++;
            deltaTg = t;
            //delay(500); //natvrdo cekej 500ms (podle pravidel) nez dojde ke zobrazeni dalsich svetel
        }
        if (b == 2 && t - deltaTg > 500) //yellow = 500
        {
            lightDo(Y1, false);
            Serial.println("Y10");
            writeBT("Y10");
            // checkBeams();
            lightDo(Y2, false);
            Serial.println("Y20");
            writeBT("Y20");
            // checkBeams();
            lightDo(Y3, false);
            Serial.println("Y30");
            writeBT("Y30");
            // checkBeams();
            lightDo(G, true);
            getPulse();
            Serial.println("GL1");
            writeBT("GL1");
            // getPulse();
            delay(green);
            lightDo(G, false);
            Serial.println("GL0");
            writeBT("GL0");
            b++;
            setSC(false);
        }
    }
    else
    {
        if (b == 1)
        { //prodleva pro srovnani x
            lightDo(Y1, true);
            writeBT("Y11");
            Serial.println("Y11");
            b++;
            deltaTg = t;
        }
        if (b == 2 && t - deltaTg > yellow) //cekej ty a rozsvit y2
        {
            if (!yellowProgress)
            {
                lightDo(Y1, false);
                writeBT("Y10");
                Serial.println("Y10");
            }
            lightDo(Y2, true);
            writeBT("Y21");
            Serial.println("Y21");
            b++;
            deltaTg = t;
        }
        if (b == 3 && t - deltaTg > yellow) // cekej ty a rozsvit y3
        {
            if (!yellowProgress)
            {
                lightDo(Y2, false);
                writeBT("Y20");
                Serial.println("Y20");
            }
            lightDo(Y3, true);
            // checkBeams();
            writeBT("Y31");
            Serial.println("Y31");
            b++;
            deltaTg = t;
        }
        if (b == 4 && t - deltaTg > yellow) //vsechno zhasni/ocekuj startovni pozici a rozsvit zbytek
        {
            // checkBeams();
            if (!yellowProgress)
            {
                lightDo(Y3, false);
                writeBT("Y30");
                Serial.println("Y30");
            }
            else
            {
                lightDo(Y1, false);
                writeBT("Y10");
                Serial.println("Y10");
                lightDo(Y2, false);
                writeBT("Y20");
                Serial.println("Y20");
                lightDo(Y3, false);
                writeBT("Y30");
                Serial.println("Y30");
            }
            lightDo(G, true);
            if (startSG)
            {
                getPulse();
                timeIsr(7, millis() - 2);
                // getPulseDown();
            }
            // timeIsr(7, millis());
            writeBT("GL1");
            Serial.println("GL1");
            deltaTg = t;
            b++;
            // if (manualStart)
            // {

            // }
        }
        if (b == 5 && t - deltaTg > green)
        {
            // delay(green);
            lightDo(G, false);
            writeBT("GL0");
            Serial.println("GL0");
            b++;

            if (beamType != NONE && (startType == SC || startType == SE))
            {
                resetBeam(beamType);
                callISR();
            }
            exitStart = true;
            // Serial.println("SA/SC completed");

            setStart(false);
        }
    }
}
void tma()
{
    // Serial.println("vsechno zhasnout");

    lightDo(99, false);
    Serial.println("NL0");
    writeBT("NL0");
    Serial.println("FL0");
    writeBT("FL0");
    Serial.println("NR0");
    writeBT("NR0");
    Serial.println("FR0");
    writeBT("FR0");

    Serial.println("Y10");
    writeBT("Y10");
    Serial.println("Y20");
    writeBT("Y20");
    Serial.println("Y30");
    writeBT("Y30");
    Serial.println("GL0");
    writeBT("GL0");
    Serial.println("RL0");
    writeBT("RL0");
    Serial.println("RR0");
    writeBT("RR0");
    exitStart = true;
    //kdyz je priblizovani tak priblizuj mode SA || SB
}
void startAll()
{
    lightDo(99, true);
    Serial.println("NL1");
    writeBT("NL1");

    Serial.println("FL1");
    writeBT("FL1");
    Serial.println("NR1");
    writeBT("NR1");
    Serial.println("FR1");
    writeBT("FR1");

    Serial.println("Y11");
    writeBT("Y11");
    Serial.println("Y21");
    writeBT("Y21");
    Serial.println("Y31");
    writeBT("Y31");
    Serial.println("GL1");
    writeBT("GL1");
    Serial.println("RL1");
    writeBT("RL1");
    Serial.println("RR1");
    writeBT("RR1");
}
void startSA(unsigned long t)
{
    // Serial.print("gsp: ");
    // Serial.println(getStartPosition());
    // Serial.print("b: ");
    // Serial.println(b);

    if (getStartPosition() || isSequence())
    { //je bud 15 nebo spustena sekvence
        // sequence = true;

        if (b == 1 && t - deltaTg > waitAfter)
        { //prodleva pro srovnani x
            // Serial.print("deltatg: ");
            // Serial.println(deltaTg);
            // Serial.print("t: ");
            // Serial.println(t);
            lightDo(Y1, true);
            writeBT("Y11");
            Serial.println("Y11");
            b++;
            deltaTg = t;
        }
        if (b == 2 && t - deltaTg > yellow) //cekej ty a rozsvit y2
        {
            if (!yellowProgress)
            {
                lightDo(Y1, false);
                writeBT("Y10");
                Serial.println("Y10");
            }
            lightDo(Y2, true);
            writeBT("Y21");
            Serial.println("Y21");
            b++;
            deltaTg = t;
        }
        if (b == 3 && t - deltaTg > yellow) // cekej ty a rozsvit y3
        {
            if (!yellowProgress)
            {
                lightDo(Y2, false);
                writeBT("Y20");
                Serial.println("Y20");
            }
            lightDo(Y3, true);
            // checkBeams();
            writeBT("Y31");
            Serial.println("Y31");
            b++;
            deltaTg = t;
        }
        if (b == 4 && t - deltaTg > yellow) //vsechno zhasni/ocekuj startovni pozici a rozsvit zbytek
        {
            checkBeams();
            if (!yellowProgress)
            {
                lightDo(Y3, false);
                writeBT("Y30");
                Serial.println("Y30");
            }
            else
            {
                lightDo(Y1, false);
                writeBT("Y10");
                Serial.println("Y10");
                lightDo(Y2, false);
                writeBT("Y20");
                Serial.println("Y20");
                lightDo(Y3, false);
                writeBT("Y30");
                Serial.println("Y30");
            }

            lightDo(G, true);
            getPulse();
            timeIsr(7, millis() - 2);
            //  getPulseDown();
            // timeIsr(7, millis());
            writeBT("GL1");
            Serial.println("GL1");

            b++;

            setFirst(true);
            setSequence(false);
            setStart(false);
        }
    }
}

void processMessage(char *m)
{
    // Serial.print("exitstart status: ");
    // Serial.println(exitStart);
    Serial.println(m);
    switch (m[0])
    {
    case 'D': //pro praci s daty
        switch (m[1])
        {
        case '0': //dotaz na data v eeprom
            getData();
            Serial.println("D$");
            writeBT("D$");
            break;
        case '1': //vymaze EEPROM
            deleteEEPROM();
            Serial.println("D00");
            writeBT("D00");
            break;
        }
        break;
    case 'G':
        startG();
        break;
    case 'S':

        switch (m[1])
        {
        case '0': //S0 vsechno zhasni
                  // startStop = true;
            // Serial.println("musi stopnout celou startovaci sekvenci pri SA!!!");
            setStart(true);
            exitStart = true;
            tma();
            manualStart = false;
            lastSA = false;
            lastSC = false;
            resetBeam(beamType);
            setStart(false);
            break;
        case '1': //S1 nech rude
            setSequence(false);
            setStart(true);
            tma();
            startR(false); //rozsvit obe rude
            startR(true);
            exitStart = true;

            manualStart = false;
            lastSA = false;
            lastSC = false;
            setStart(false);
            break;
        case '2': //S2 nedela nic ???

            switch (m[2])
            {
            case '1':
                //rozsvitVsechno
                setStart(true);
                startAll();
                setStart(false);
                break;
            }
            break;
        // case '4': //S4 ???
        //     //
        //     break;
        case 'A': //SA startuj - proste posle z pc signal ke startovani
                  //druhstartu Drag
                  //rozsvit priblizovaci
            IS = true;
            beamType = DRAG;
            setStartPosition(0);
            b = 1;
            setFirst(true);
            setStart(true);
            tma();
            resetBeam(beamType);
            // Serial.println("SA case");
            startTrue = true;
            manualStart = false;
            lastSC = false;
            startSG = false;
            lastSA = true;
            break;
        case 'B':

            beamType = TMS;
            resetBeam(TMS); //priblizuj TMS
            //rozsvit Priblizovaci
            lastSA = false;
            lastSC = false;
            startSG = false;
            break;
        case 'C':
            //startSC(); // pusti startovaci sekvenci na konci if beamType != NONE; priblizuj();
            setStart(true);
            startType = SC;
            manualStart = false;
            lastSA = false;
            lastSC = true;
            b = 1;
            // start(millis());

            break;
        case 'D': //startuj RG
            beamType = NONE;
            lastSA = false;
            lastSC = false;
            //neni nutne nic delat, vse resi signaly startovni fronty

            break;
        case 'E': //SE rucni start, ve fronte posle signal S0
            setStart(false);
            manualStart = true;
            lastSA = false;
            lastSC = false;
            break;
        case 'F': //SF - kary, pak to haze SC nebo SF
            setStart(true);
            beamType = NONE; //nepouzivej priblizovaci
            resetBeam(beamType);
            //startuj kary, jede v tom s SC
            manualStart = false;
            lastSA = false;
            lastSC = false;
            break;
        case 'G':
            beamType = DRAG;
            resetBeam(DRAG);
            manualStart = false;
            lastSA = false;
            lastSC = false;
            startSG = true;
            break;
        }

        break;
    case 'R':

        lastSA = false;
        lastSC = false;
        switch (m[1])
        {
        case '0':
            stopRG();
            break;

        case '1':
            startR(true);
            break;
        case '2':
            startR(false);
            break;
        case '3':
            startR(false);
            startR(true);
            break;
        case '4':
            startG();
            break;
        }
        break;
    case 'V':

        lastSA = false;
        lastSC = false;
        switch (m[1])
        {
        case 'A':
            yellowProgress = true; //pridava
            break;
        case 'B':
            yellowProgress = false; //postupuje
            break;
        }
        break;
    case 'T':

        lastSA = false;
        lastSC = false;
        waitAfter = (int)(m[1] - 48) * 10;
        yellow = (int)(m[2] - 48) * 10;
        green = (int)(m[3] - 48) * 10;
        yellow == 0 ? (yPro = true) : (yPro = false);
        // Serial.print("waitafter:\t");
        // Serial.println(waitAfter);
        // Serial.print("zluta:\t");
        // Serial.println(yellow);
        // Serial.print("zelena:\t");
        // Serial.println(green);
        break;
    case 'I':

        if (lastSA || lastSC)
            switch (m[1])
            {
            case 'S':
                exitStart = false;
                IS = true;
                //lastIS = true;
                //tma();
                break;
            case 'L': //interrupt leva
                if (!IS)
                    startR(true);
                break;
            case 'R': //interrupt leva
                if (!IS)
                    startR(false);
                break;
            default:
                // exitStart = false;
                tma();
                break;
            }

        break;
    case 0x02: //jediny co zacina stx# je synchro prikaz
        //0x02#WC 007 02 12:42 21/10/2020 je to prikaz z protokolu TAGHEUER
        String string = "";
        string = convertToString(m, strlen(m));
        // Serial.println(ss);
        int hour = string.substring(12, 14).toInt();
        int minutes = string.substring(15, 17).toInt();
        setSyncReady(true, hour, minutes);
        startR(false);
        startR(true);
        // a = h *x na tisiciny
        //     b = m * y na tisiciny;;
        // timeSync = a+b;
        // Serial.println("synchro ready");
        // Serial.print(hh);
        // Serial.print(":");
        // Serial.println(mm);
        break;
    }
}
