/*
Pro praci s casovymi prerusenimi

@author = Vojtech Hajek
DP Casomerna jednotka pro automobilovy sprint
*/

#include "tisr.h"
#include <Arduino.h>
#include "I2C.h"
#include "myBluetooth.h"

#define i0Pin 33
#define i1Pin 4
#define i2Pin 2
#define i3Pin 12

unsigned int timeSync;       //casova konstanta v tisicinach cas ze synchronizace
unsigned int timeSyncMillis; //doba, kdy doslo k synchronizaci
unsigned int debounce;       //po protnuti bude chvili kanal neobsluhovany //ovladat po Seriovce??
bool syncReady;
bool isrChange;
bool startSequence;
unsigned long timeBuffer[40]; //na 20 casu, melo by stacit po poslani na seriovku se ukladaji do eeprom;
int k;                       //pro pohyb v bufferu
int h;                       //hod
int m;                       //min po synchro
int i0pre;  //casy minulych preruseni
int i1pre;
int i2pre;
int i3pre;

void initTisr()
{
    pinMode(i0Pin, INPUT_PULLUP);
    pinMode(i1Pin, INPUT_PULLUP);
    pinMode(i2Pin, INPUT_PULLUP);
    pinMode(i3Pin, INPUT_PULLUP);

    attachInterrupt(i0Pin, i0Isr, FALLING);
    attachInterrupt(i1Pin, i1Isr, FALLING);
    attachInterrupt(i2Pin, i2Isr, FALLING);
    attachInterrupt(i3Pin, i3Isr, FALLING);

    timeSyncMillis = 0;
    timeSync = 0;
    syncReady = false;
    isrChange = false;
    startSequence = false;
    debounce = 200;
    k = 0;
    h = 0;
    m = 0;
    i0pre = 0;
    i1pre = 0;
    i2pre = 0;
    i3pre = 0;
   
}
void setStart(bool s)
{
    startSequence = s;
}
void setHM(int hh, int mm)
{
    h = hh;
    m = mm;
    timeSync = h * 3600000 + m * 60000;
    lightDo(98, true); //rozsviti cervene
    Serial.print("S$");
    // Serial.println(timeSync);
}
void setSyncReady(bool s, int h, int m)
{
    syncReady = s;
    setHM(h, m);
}
void timeIsr(int input, unsigned long t)
{
    if (syncReady)
    {
        timeSyncMillis = t;
        syncReady = false;
        lightDo(98, false);
    }
    else
    {
        //   unsigned int t = millis();
        unsigned int tt = t - timeSyncMillis; //cisty cas preruseni v millis
        timeBuffer[k % 40] = tt + timeSync;   //cas + konstanta synchrnizace
        timeBuffer[(k + 1) % 40] = input;     //preruseni
        k += 2;
        isrChange = true;
    }
}

void IRAM_ATTR i0Isr()
{
    int t = millis();
    if (t - i0pre > debounce)
    {
        timeIsr(0, t);
        i0pre = t;
    }
}
void IRAM_ATTR i1Isr()
{
    int t = millis();
    if (t - i1pre > debounce)
    {
        timeIsr(1, t);
        i1pre = t;
    }
}
void IRAM_ATTR i2Isr()
{
    int t = millis();
    if (t - i2pre > debounce)
    {
        timeIsr(2, t);
        i2pre = t;
    }
}
void IRAM_ATTR i3Isr()
{
    int t = millis();
    if (t - i3pre > debounce)
    {
        timeIsr(3, t);
        i3pre = t;
    }
}
bool isStart()
{
    return startSequence;
}
void convertMillisToTime(unsigned long time, int isr)
{
    // 60*60*24*1000
    // Serial.print("cas: ");
    // Serial.println(time);
    //22h * 60s*60m *1000t
    int hh = ((time / (1000 * 60 * 60)) % 24);
    int mm = ((((time / 1000) / 60)) % 60);
    int ss = (time / 1000) % 60;
    int ttt = time % 1000;

    // Serial.print("hodina");
    // Serial.println(hh);
    // Serial.print("min");
    // Serial.println(mm);
    // Serial.print("ss");
    // Serial.println(ss);
    // Serial.print("ttt");
    // Serial.println(ttt);
    String sHh = "M";
    sHh += isr;
    sHh += "    ";

    if (hh == 0)
        sHh += "00";
    else if (hh < 10)
    {
        sHh += "0";
        sHh += hh;
    }
    else
        sHh += hh;
    sHh += ":";
    if (mm == 0)
        sHh += "00";
    else if (mm < 10)
    {
        sHh += "0";
        sHh += mm;
    }
    else
        sHh += mm;
    sHh += ":";
    if (ss == 0)
        sHh += "00";
    else if (ss < 10)
    {
        sHh += "0";
        sHh += ss;
    }
    else
        sHh += ss;
    sHh += ".";
    if (ttt == 0)
        sHh += "000";
    else if (ttt < 10)
    {
        sHh += "00";
        sHh += ttt;
    }
    else if (ttt < 100)
    {
        sHh += "0";
        sHh += ttt;
    }
    else
        sHh += ttt;
    // zprava ve formatu odeslana po seriovce a bt
    //Mi____hh:mm:ss.ttt
    Serial.println(sHh);
    writeBT(sHh);
}

void checkIsr()
{
    //je zmena na preruseni a zaroven se nestartuje kvuli kolizi na I2C
    if (isrChange && !startSequence && !isDataRead())
    {

        for (int i = 0; i < sizeof(timeBuffer) / sizeof(unsigned int); i += 2)
        {
            if (timeBuffer[i] != 0)
            {
                convertMillisToTime(timeBuffer[i], timeBuffer[i + 1]);
                saveData((timeBuffer[i] << 4) | timeBuffer[i + 1]);
                timeBuffer[i] = 0; //nastav do casu nulu, takovy cas byt nemuze
                //cas v millis s prerusenim
            }
            timeBuffer[i] = 0;
        }

        isrChange = false;
    }
}
